import React, { Fragment } from 'react'
import qs from 'querystring'
import PropTypes from 'prop-types'
import { connect } from 'react-redux'
import { Spinner } from '@pubsweet/ui'
import { compose, withHandlers, withState } from 'recompose'
import {
  getUserToken,
  currentUserIsReviewer,
} from 'pubsweet-component-faraday-selectors'

const ZipFiles = ({ disabled, fetching, children, downloadFiles }) => (
  <Fragment onClick={!disabled ? downloadFiles : null}>
    {fetching ? <Spinner /> : children}
  </Fragment>
)

const cache = {}

const reviewerFiles = ['manuscripts', 'supplementary']
const defaultFiles = [...reviewerFiles, 'coverLetter']

const DownloadZipFiles = compose(
  connect((state, { collectionId }) => ({
    token: getUserToken(state),
    isReviewer: currentUserIsReviewer(state, collectionId),
  })),
  withState('fetching', 'setFetching', false),
  withHandlers({
    createAnchorElement: () => (file, filename) => {
      const url = URL.createObjectURL(file)
      const a = document.createElement('a')

      a.href = url
      a.download = filename
      document.body.appendChild(a)

      return {
        a,
        url,
      }
    },
    removeAnchorElement: () => (a, url) => {
      document.body.removeChild(a)
      URL.revokeObjectURL(url)
    },
  }),
  withHandlers({
    downloadFiles: ({
      token,
      isReviewer,
      fragmentId,
      setFetching,
      archiveName,
      createAnchorElement,
      removeAnchorElement,
    }) => () => {
      const getUrl = `${
        window.location.origin
      }/api/files/${fragmentId}?${qs.stringify({
        fileTypes: isReviewer ? reviewerFiles : defaultFiles,
      })}`
      if (cache[fragmentId]) {
        const fileName = archiveName || `${fragmentId}-archive.zip`

        const { a, url } = createAnchorElement(cache[fragmentId], fileName)
        a.click()
        removeAnchorElement(a, url)
      } else {
        setFetching(fetching => true)
        const xhr = new XMLHttpRequest()
        xhr.onreadystatechange = function onXhrStateChange() {
          if (this.readyState === 4) {
            setFetching(fetching => false)
            if (this.status >= 200 && this.status < 300) {
              const fileName = archiveName || `${fragmentId}-archive.zip`
              const f = new File([this.response], fileName, {
                type: 'application/zip',
              })
              cache[fragmentId] = f

              const { a, url } = createAnchorElement(f, fileName)
              a.click()
              removeAnchorElement(a, url)
            }
          }
        }
        xhr.open('GET', getUrl)
        xhr.responseType = 'blob'
        xhr.setRequestHeader('Authorization', `Bearer ${token}`)
        xhr.send()
      }
    },
  }),
)(ZipFiles)

DownloadZipFiles.propTypes = {
  disabled: PropTypes.bool,
  archiveName: PropTypes.string,
  fragmentId: PropTypes.string.isRequired,
}

DownloadZipFiles.defaultProps = {
  disabled: false,
}

export default DownloadZipFiles
